#ifndef _Polygon3_h__
#define _Polygon3_h__


//#include <iostream>
//#include <math.h>
//#include "common.h"
#include "Containers.h"
#include "Mesh.h"

class Polygon3;
class Plane3;
class Face;


/** Queue storing a soup of polygons used during BSP tree construction
*/
typedef queue<Polygon3 *> PolygonQueue;

/** Class representing a general planar polygon in 3d.
*/
class Polygon3
{
public:
	Polygon3();
	Polygon3(const VertexContainer &vertices);

	/** Copies all the vertices of the face.
	*/
	Polygon3(Face *face, Mesh *parent);
	
	/** Returns supporting plane of this polygon.
	*/
	Plane3 GetSupportingPlane();

	/** Splits polygon.
		@param partition the split plane
		@param front the front polygon
		@param back the back polygon
		@param splits number of splits
	*/
	void Split(Plane3 *partition, Polygon3 *front, Polygon3 *back, int &splits);

	enum {BACK_SIDE, FRONT_SIDE, SPLIT, COINCIDENT};

	/** Side of the plane where the polygon is located.
	    @returns one of BACK_SIDE, FRONT_SIDE, SPLIT, COINCIDENT
	*/
	int Side(Plane3 *plane);

	/** Deletes all polygons om the queue.
	*/
	static void DeletePolygons(PolygonQueue *polys);

	/// vertices are connected in counterclockwise order.
	VertexContainer mVertices;
};

// Overload << operator for C++-style output
inline ostream&
operator<< (ostream &s, const Polygon3 &A)
{
	VertexContainer::const_iterator it;

	s << "Polygon:\n";
	
	for (it = A.mVertices.begin(); it != A.mVertices.end(); ++it)
		s << *it << endl;
	
	return s;
}


#endif
