#include "SceneGraph.h"
#include "Exporter.h"
#include "UnigraphicsParser.h"
#include "X3dParser.h"
#include "Preprocessor.h"
#include "ViewCell.h"
#include "Environment.h"

  
bool
Preprocessor::LoadViewcells(const string filename)
{
  
  return true;
}

bool
Preprocessor::GenerateViewcells()
{
	return BuildBspTree();
}

bool
Preprocessor::LoadScene(const string filename)
{
  // use leaf nodes of the original spatial hiearrchy as occludees

  mSceneGraph = new SceneGraph;

  
  Parser *parser;

  if (strstr(filename.c_str(), ".x3d"))
    parser = new X3dParser;
  else
    parser = new UnigraphicsParser;

  bool result = parser->ParseFile(filename, &mSceneGraph->mRoot);
  
  return result;
}

bool
Preprocessor::ExportPreprocessedData(const string filename)
{
  return false;
}

bool
Preprocessor::BuildKdTree()
{
  mKdTree = new KdTree;
  // add mesh instances of the scene graph to the root of the tree
  KdLeaf *root = (KdLeaf *)mKdTree->GetRoot();
  mSceneGraph->CollectObjects(&root->mObjects);
  
  mKdTree->Construct();
  return true;
}

bool
Preprocessor::BuildBspTree()
{
	ObjectContainer objects;
	mSceneGraph->CollectObjects(&objects);

	mBspTree = new BspTree();

	char constructionMethodStr[64];

	environment->GetStringValue("BspTree.constructionMethod", constructionMethodStr);

	int constructionMethod = BspTree::VIEWCELLS;
	
	if (strcmp(constructionMethodStr, "viewCells") == 0)
		constructionMethod = BspTree::VIEWCELLS;
	else if (strcmp(constructionMethodStr, "sceneGeometry") == 0)
		constructionMethod = BspTree::SCENE_GEOMETRY;
	else 
	{
		cerr << "Wrong bsp construction method " << constructionMethodStr << endl;
		exit(1);
    }

	switch (constructionMethod)
	{
	case BspTree::VIEWCELLS:

		mViewcells.clear();
		
		// derive viewcells from the scene objects
		ViewCell::DeriveViewCells(objects, mViewcells, 1000);
		
		mBspTree->Construct(mViewcells);
		break;
	case BspTree::SCENE_GEOMETRY:
		mBspTree->Construct(objects);
		break;
	default:
		break;
	}
	return true;
}


void
Preprocessor::KdTreeStatistics(ostream &s)
{
  s<<mKdTree->GetStatistics();
}

void
Preprocessor::BspTreeStatistics(ostream &s)
{
	s << mBspTree->GetStatistics();
}

bool
Preprocessor::Export( const string filename,
		      const bool scene,
		      const bool kdtree
		      )
{
  Exporter *exporter = Exporter::GetExporter(filename);

  if (exporter) {
    if (scene)
      exporter->ExportScene(mSceneGraph->mRoot);

    if (kdtree) {
      exporter->SetWireframe();
      exporter->ExportKdTree(*mKdTree);
    }

    delete exporter;
    return true;
  }

  return false;
}



